############################################################
# R script illustrating the use of RSiena's simulation
# facilities. 
# version of 4 February 2017
# written by Christian Steglich
############################################################

# Consideration to begin with:
# - has the zip-archive been unpacked?
# - do you need to set the working directory?

# load data set:
load("klas03edata.RData")
ls()

# load RSiena commands to the R namespace:
library(RSiena)

# check installed version:
packageVersion("RSiena")

# prepare data for RSiena (just the middle data period!):
thedata <- sienaDataCreate(
	friendship = sienaDependent(array(
		c(friendship.net2,friendship.net3),
		dim=c(dim(friendship.net2),2))),
	sex.M = coCovar(demographics[,1],centered=FALSE),
	primary = coDyadCovar(primary.net)
)
thedata

# specify a simple model:
themodel <- getEffects(thedata)
themodel <- includeEffects(themodel,X,interaction1='primary')
themodel <- includeEffects(themodel,simX,interaction1='sex.M') # use this row for "simX" specification*
#themodel <- includeEffects(themodel,sameX,interaction1='sex.M') # use this row for "sameX" specification*
themodel <- includeEffects(themodel,transTrip,transRecTrip)
themodel
# * consult workshop slides for the difference between the two specifications

# estimate the model:
estimation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567)
theresults <- siena07(estimation.options,
	data=thedata,effects=themodel)
theresults

# How many simulation do we want to work with per scenario?
N <- 1000

# generate N simulated networks for the end of the period
# under study (when "friendship.net3" was measured):
?sienaAlgorithmCreate
simulation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567,
	nsub=0,n3=N)
?siena07
simulation.results <- siena07(simulation.options,
	data=thedata,effects=themodel,
	prevAns=theresults,returnDeps=TRUE)
simulation.results
# Note that standard errors and convergence are unreliable here!

# Where are the simulated networks?
simulation.results$sims

# The n'th simulated network is stored as an edgelist under 
# "simulation.results$sims[[n]]$Data1$friendship$`1`":
simulation.results$sims[[1]]$Data1$friendship$`1`  # first simulated network
simulation.results$sims[[11]]$Data1$friendship$`1` # 11th simulated network, etc.

# let us use a function that extracts the n'th network into an adjacency matrix:
source('extract-simulations.R')
extract(simulation.results,1)  # first simulated network
extract(simulation.results,11) # 11th simulated network, etc.


# How much do simulated networks resemble the observed network?
# For this aim, we make use of the R-package "sna":
library(sna) # install first if necessary

# PLOTTING:

	# make graphical window three plots wide:
	par(mfrow=c(1,3))

	# plot empirical network and two simulated ones:
	gplot(friendship.net3,vertex.col=demographics[,1],main='empirical')
	gplot(extract(simulation.results,1),vertex.col=demographics[,1],main='1st simulated')
	gplot(extract(simulation.results,11),vertex.col=demographics[,1],main='11th simulated')

	# close graphical window:
	dev.off()

# STATISTICS: (this requires a tiny bit more of R programming)

	# FIRST EXAMPLE: number of ties in the networks
	# extract number of ties for all simulated networks:
	ties <- unlist(lapply(
		lapply(1:N,function(n){extract(simulation.results,n)}),
		sum 
	))
	ties
	
	# plot distribution:
	plot(density(ties))
	
	# add empirically observed value as vertical line:
	obs.ties <- sum(friendship.net3)
	obs.ties
	lines(rep(obs.ties,2),c(0,0.05),col='red')

	# SECOND EXAMPLE: reciprocity index of the networks
	# extract percentage of reciprocated ties for all simulated networks:
	reciprocity <- unlist(lapply(
		lapply(1:N,function(n){extract(simulation.results,n)}),
		grecip,measure='edgewise' 
	))
	round(reciprocity,3)
	
	# plot distribution:
	plot(density(reciprocity))
	
	# add empirically observed value as vertical line:
	obs.reciprocity <- grecip(friendship.net3,measure='edgewise')
	obs.reciprocity
	lines(rep(obs.reciprocity,2),c(0,10),col='red')

# UNDERSTANDING RSiena CONVERGENCE DIAGNOSTICS:
	
	# recall convergence diagnostics based on simulated data:
	simulation.results
	# last column indicates z-scores of empirically observed
	# effect statistics in distribution of simulated statistics
	
	# check with number-of-ties example calculated above:
	(obs.ties - mean(ties)) / sd(ties)
	# is indeed the "outdegree" convergence indicator
	
	# check with percentage of reciprocated ties:
	(obs.reciprocity - mean(reciprocity)) / sd(reciprocity)
	# is NOT "reciprocity" convergence indicator
	# Why not? -> Because effect statistic is not PERCENTAGE
	#             but NUMBER of reciprocal ties!
	
	# check with number of reciprocated ties:
	# (number = percentage*total)
	(obs.reciprocity*obs.ties - mean(reciprocity*ties)) / sd(reciprocity*ties)
	# is indeed "reciprocity" convergence indicator
	

# Finally let us use RSiena as agent-based simulation tool...

# Suppose we can manipulate the strength of the sex homophily parameter
# "same sex.M", from minus its estimated size to twice its estimated size.
# How do simulated networks then look like?

# Where can the estimate be found?
theresults$theta
# it's the last (=seventh) in this row that we wish to manipulate

# manipulate from minus its estimated size to twice its estimated size:
(manip.homophily <- c(-2,-1,0,1,2,3)*theresults$theta[[7]])
# could be more fine-grained, but okay...

# put these values into RSiena objects that can be used for simulations:
manip.models <- rep(list(theresults),length(manip.homophily))
manip.models <- lapply(1:length(manip.homophily),function(i){
	m <- manip.models[[i]]
	m$theta[[7]] <- manip.homophily[i]
	return(m)
})
manip.models

# simulate N networks under each condition:
newsims <- lapply(manip.models,function(m){
	siena07(simulation.options,
	data=thedata,effects=themodel,
	prevAns=m,returnDeps=TRUE)
})


# make graphical window four plots wide:
par(mfrow=c(1,length(manip.homophily)))

# plot series of 100th-simulated networks:
lapply(1:length(newsims),function(i){
	gplot(extract(newsims[[i]],100),vertex.col=demographics[,1],
	main=paste('factor',manip.homophily[i]))
})
gplot(extract(newsims[[1]],100),vertex.col=demographics[,1],main='factor -2')
gplot(extract(newsims[[2]],100),vertex.col=demographics[,1],main='factor -1')
gplot(extract(newsims[[3]],100),vertex.col=demographics[,1],main='factor 0')
gplot(extract(newsims[[4]],100),vertex.col=demographics[,1],main='factor +1')
gplot(extract(newsims[[5]],100),vertex.col=demographics[,1],main='factor +2')
gplot(extract(newsims[[4]],100),vertex.col=demographics[,1],main='factor +3')

# close graphical window:
dev.off()


gender <- demographics[,1]
crossgender.possible <- 1*outer(gender,gender,"!=")
netlist1 <- lapply(1:N,function(i)extract(newsims[[1]],i))
netlist2 <- lapply(1:N,function(i)extract(newsims[[2]],i))
netlist3 <- lapply(1:N,function(i)extract(newsims[[3]],i))
netlist4 <- lapply(1:N,function(i)extract(newsims[[4]],i))
netlist5 <- lapply(1:N,function(i)extract(newsims[[5]],i))
netlist6 <- lapply(1:N,function(i)extract(newsims[[6]],i))
ccgt <- matrix(NA,nc=6,nr=N)
for (i in 1:N) {
	ccgt[i,1] <- sum(netlist1[[i]]*crossgender.possible)
	ccgt[i,2] <- sum(netlist2[[i]]*crossgender.possible)
	ccgt[i,3] <- sum(netlist3[[i]]*crossgender.possible)
	ccgt[i,4] <- sum(netlist4[[i]]*crossgender.possible)
	ccgt[i,5] <- sum(netlist5[[i]]*crossgender.possible)
	ccgt[i,6] <- sum(netlist6[[i]]*crossgender.possible)
}

boxplot(ccgt,xaxt='n')
lines(c(0.5,6.5),
	rep(sum(friendship.net3*crossgender.possible),2),
	col='red',lty="dashed",lwd=2)
axis(1,1:6,-2:3)

# or the same as violin plot:
require(vioplot)
vioplot(data.frame(ccgt),names=-2:3,col='lightgrey')
lines(c(0.5,6.5),
	rep(sum(friendship.net3*crossgender.possible),2),
	col='red',lty="dashed",lwd=2)

# now make plots for sensitivity of number of ties to manipulation:
ties <- matrix(NA,nc=6,nr=N)
for (i in 1:N) {
	ties[i,1] <- sum(netlist1[[i]])
	ties[i,2] <- sum(netlist2[[i]])
	ties[i,3] <- sum(netlist3[[i]])
	ties[i,4] <- sum(netlist4[[i]])
	ties[i,5] <- sum(netlist5[[i]])
	ties[i,6] <- sum(netlist6[[i]])
}

vioplot(data.frame(ties),names=-2:3,col='lightgrey')
lines(c(0.5,6.5),
	rep(sum(friendship.net3),2),
	col='red',lty="dashed",lwd=2)

# save results if needed later:
save.image("Type 1 simulation lab results.RData")
