############################################################
# R script illustrating the use of RSiena's simulation
# facilities. 
# version of 10 March 2020
# written by Christian Steglich
############################################################

# This "Type 2" script does the same as the "Type 1" 
# script, but re-calibrates all parameters (except the
# manipulated one) anew to the data set.

# load data set:
load("klas03edata.RData")
ls()

# load RSiena commands to the R namespace:
library(RSiena)

# check installed version:
packageVersion("RSiena")

# prepare data for RSiena (just the middle data period!):
thedata <- sienaDataCreate(
	friendship = sienaDependent(array(
		c(friendship.net2,friendship.net3),
		dim=c(dim(friendship.net2),2))),
	sex.M = coCovar(demographics[,1],centered=FALSE),
	primary = coDyadCovar(primary.net)
)
thedata

# specify a simple model:
themodel <- getEffects(thedata)
themodel <- includeEffects(themodel,X,interaction1='primary')
themodel <- includeEffects(themodel,simX,interaction1='sex.M')
themodel <- includeEffects(themodel,transTrip,transRecTrip)
themodel
# NOTE that difference between simX and sameX should NOT MATTER for Type 2 simulations!

# estimate the model:
estimation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567)
theresults <- siena07(estimation.options,
	data=thedata,effects=themodel)
theresults

# How many simulation do we want to work with per scenario?
N <- 1000

# import the function that extracts the n'th network into an adjacency matrix:
source('extract-simulations.R')

# Again manipulate the strength of the sex homophily parameter from
# minus its estimated size to twice its estimated size:
(manip.homophily <- c(-2,-1,0,1,2,3)*theresults$theta[[7]])
# could be more fine-grained, but okay...

# put these values into RSiena objects that can be used for simulations:
manip.models <- rep(list(theresults),length(manip.homophily))
manip.models <- lapply(1:length(manip.homophily),function(i){
	m <- manip.models[[i]]
	m$theta[[7]] <- manip.homophily[i]
	return(m)
})
manip.models

# make sure manipulated effect is "fixed", i.e., will not be estimated:
themodel <- setEffect(themodel,simX,interaction1='sex.M',fix=TRUE)
		
# Re-calibrate all other parameters by the estimation algorithm:
calib.models <- lapply(manip.models,function(mm){
	cm <- siena07(estimation.options,
		data=thedata,effects=themodel,prevAns=mm)
		# the prevAns option makes sure that the manipulated
		# specification will be used.
	return(cm)
})
calib.models


# simulate N networks under each condition:
simulation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567,
	nsub=0,n3=N)
calsims <- lapply(calib.models,function(m){
	siena07(simulation.options,
	data=thedata,effects=themodel,
	prevAns=m,returnDeps=TRUE)
})


# make graphical window four plots wide:
par(mfrow=c(1,length(manip.homophily)))

# plot series of 100th-simulated networks:
require(sna)
lapply(1:length(calsims),function(i){
	gplot(extract(calsims[[i]],100),vertex.col=demographics[,1],
	main=paste('factor',manip.homophily[i]))
})
gplot(extract(calsims[[1]],100),vertex.col=demographics[,1],main='factor -2')
gplot(extract(calsims[[2]],100),vertex.col=demographics[,1],main='factor -1')
gplot(extract(calsims[[3]],100),vertex.col=demographics[,1],main='factor 0')
gplot(extract(calsims[[4]],100),vertex.col=demographics[,1],main='factor +1')
gplot(extract(calsims[[5]],100),vertex.col=demographics[,1],main='factor +2')
gplot(extract(calsims[[4]],100),vertex.col=demographics[,1],main='factor +3')
# looks very much akin to the results in Type 1 simulations

# close graphical window:
dev.off()


gender <- demographics[,1]
crossgender.possible <- 1*outer(gender,gender,"!=")
netlist1 <- lapply(1:N,function(i)extract(calsims[[1]],i))
netlist2 <- lapply(1:N,function(i)extract(calsims[[2]],i))
netlist3 <- lapply(1:N,function(i)extract(calsims[[3]],i))
netlist4 <- lapply(1:N,function(i)extract(calsims[[4]],i))
netlist5 <- lapply(1:N,function(i)extract(calsims[[5]],i))
netlist6 <- lapply(1:N,function(i)extract(calsims[[6]],i))
ccgt <- matrix(NA,nc=6,nr=N)
for (i in 1:N) {
	ccgt[i,1] <- sum(netlist1[[i]]*crossgender.possible)
	ccgt[i,2] <- sum(netlist2[[i]]*crossgender.possible)
	ccgt[i,3] <- sum(netlist3[[i]]*crossgender.possible)
	ccgt[i,4] <- sum(netlist4[[i]]*crossgender.possible)
	ccgt[i,5] <- sum(netlist5[[i]]*crossgender.possible)
	ccgt[i,6] <- sum(netlist6[[i]]*crossgender.possible)
}

boxplot(ccgt,xaxt='n')
lines(c(0.5,6.5),
	rep(sum(friendship.net3*crossgender.possible),2),
	col='red',lty="dashed",lwd=2)
axis(1,1:6,-2:3)
# also here: looks very similar

# or the same as violin plot:
require(vioplot)
vioplot(data.frame(ccgt),names=-2:3,col='lightgrey')
lines(c(0.5,6.5),
	rep(sum(friendship.net3*crossgender.possible),2),
	col='red',lty="dashed",lwd=2)

# now make plots for sensitivity of number of ties to manipulation:
ties <- matrix(NA,nc=6,nr=N)
for (i in 1:N) {
	ties[i,1] <- sum(netlist1[[i]])
	ties[i,2] <- sum(netlist2[[i]])
	ties[i,3] <- sum(netlist3[[i]])
	ties[i,4] <- sum(netlist4[[i]])
	ties[i,5] <- sum(netlist5[[i]])
	ties[i,6] <- sum(netlist6[[i]])
}

vioplot(data.frame(ties),names=-2:3,col='lightgrey')
lines(c(0.5,6.5),
	rep(sum(friendship.net3),2),
	col='red',lty="dashed",lwd=2)
# and here we see the difference! This simulation approach
# makes sure that the number of ties is kept at the 
# observed value!

# The same holds for the number of reciprocated ties, the 
# number of transitive triplets, and the statistics
# corresponding to the remaining parameters except the
# manipulated one.

# save results if needed later:
save.image("Type 2 simulation lab results.RData")
