############################################################
# R script illustrating the use of RSiena's simulation
# facilities. 
# version of 11 March 2020
# written by Christian Steglich
############################################################

# This script illustrates "Type 3" simulation studies
# in which the input data sets are manipulated.

# load data set:
load("klas03edata.RData")
ls()

# load RSiena commands to the R namespace:
library(RSiena)

# check installed version:
packageVersion("RSiena")

# As calibration scenario, we estimate a model that
# evolves a random network into the one observed
# at time point 3.

# Question: Why take a random network and not an obderved one?

# Answer: Because observed networks already are very gender
#         segregated, they are bad starting points for
#         studying the contributing factors for segregation.
#
#         This is similar to explaing a variable with low variance:
#         if there is little change necessary to obtain high
#         segregation levels

# make a random network:
require(sna)
set.seed(4321)
randomnet <- rgraph(n=nrow(friendship.net2),
	tprob=gden(friendship.net2))

# generate network evolution dataset for RSiena:
thedata <- sienaDataCreate(
	friendship = sienaDependent(array(
		c(randomnet,friendship.net3),
		dim=c(dim(friendship.net2),2))),
	sex.M = coCovar(demographics[,1],centered=TRUE),
	primary = coDyadCovar(primary.net)
)
thedata

# specify a simple model:
themodel <- getEffects(thedata)
themodel <- includeEffects(themodel,X,interaction1='primary')
themodel <- includeEffects(themodel,egoX,altX,simX,interaction1='sex.M')
themodel <- includeEffects(themodel,gwespFF)
themodel <- includeInteraction(themodel,gwespFF,recip)
themodel

# estimate the model:
estimation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567)
theresults <- siena07(estimation.options,
	data=thedata,effects=themodel)
theresults


# We now manipulate the gender variable - this is a bit
# complicated because we need to retain the original gender 
# variable (observed network structure is aligned with this
# variable), but need to embed it into a sequence of scenarios
# ranging from all-male to all-female.

# get gender info:
sexM <- demographics[,1]
n <- length(sexM)
# make random order of first males, then females:
swaporder <- c(sample(which(sexM==2)),sample(which(sexM==1)))
# The idea is now that we start with an all-female vector
# and successively swap gender of the individuals in this
# sequence. After all males have swapped, we have the empirical
# gender variable. Then the females swap until we have an
# all-male vector.
(dataversions <- lapply(1:n,function(s){
	gnd <- rep(1,n) # make gender vector "all female"
	gnd[swaporder[1:s]] <- 2 # turn male in designated order
	return(gnd)
}))
# contains increasingly many males, the empirical sexM variable
# is embedded in row 14; proof:
sapply(dataversions,function(d){sum(d==sexM)})


# thin the scenarios a bit (we now have 26 of them, takes a lot of time):
(dataversions <- dataversions[3*(1:8)]) 
# we retain 8 scenarios


# make RSiena data objects with them:
inputdata <- lapply(dataversions,function(dv){
	return(sienaDataCreate(
		friendship = sienaDependent(array(
			c(randomnet,friendship.net3),
			dim=c(dim(friendship.net2),2))),
		sex.M = coCovar(dv,centered=TRUE),
		primary = coDyadCovar(primary.net)
	))
})
inputdata

# How many simulation do we want to work with per scenario?
N <- 1000

# import the function that extracts the n'th network into an adjacency matrix:
source('extract-simulations.R')

# simulate N networks under each condition:
simulation.options <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='results',cond=FALSE,seed=1234567,
	nsub=0,n3=N)
type3sims <- lapply(inputdata,function(d){
	siena07(simulation.options,
	data=d,effects=themodel,
	prevAns=theresults,returnDeps=TRUE)
})
# Note that there is a warning about the three gender
# effects (egoX, altX and simX) being collinear. This
# probably occured in a scenario in which one gender
# group is very small.


# calculate number of cross-gender ties per simulation:
ccgt <- lapply(1:length(inputdata),function(i){
	gender <- dataversions[[i]]
	crossgender.possible <- 1*outer(gender,gender,"!=")
	netlist <- sapply(1:N,function(j){
		nl <- extract(type3sims[[i]],j)
		return(sum(nl*crossgender.possible))
	})
})

# plot how this number looks like per scenario:
boxplot(ccgt,xaxt='n',xlab="fraction females",main='#cross-gender ties')
crossg <- 1*outer(sexM,sexM,"!=")
lines(c(0.5,22.5),
	rep(sum(friendship.net3*crossg),2),
	col='red',lty="dashed",lwd=2)
axis(1,1:length(inputdata),sapply(lapply(dataversions,table),prop.table)[1,])
# Shows strong opportunity dependence of cross-gender ties.


# study other outcome, calculate transitivity per simulation:
trans <- lapply(1:length(inputdata),function(i){
	require(sna)
	netlist <- sapply(1:N,function(j){
		nl <- extract(type3sims[[i]],j)
		return(gtrans(nl))
	})
})


# This time as violins:
require(vioplot)
vioplot(data.frame(trans),xlab="fraction females",main='fraction transitive triplets',
	names=sapply(lapply(dataversions,table),prop.table)[1,])
lines(c(0,23),
	rep(gtrans(friendship.net3),2),
	col='red',lty="dashed",lwd=2)


# study other outcome, calculate density per simulation:
dens <- lapply(1:length(inputdata),function(i){
	require(sna)
	netlist <- sapply(1:N,function(j){
		nl <- extract(type3sims[[i]],j)
		return(gden(nl))
	})
})


# Again as violins:
require(vioplot)
vioplot(data.frame(dens),xlab="fraction females",main='network density',
	names=sapply(lapply(dataversions,table),prop.table)[1,])
lines(c(0,23),
	rep(gden(friendship.net3),2),
	col='red',lty="dashed",lwd=2)


# save results if needed later:
save.image("Type 3 simulation lab results.RData")
